# Import required libraries
import torch
from pytorch_transformers import GPT2Tokenizer, GPT2LMHeadModel

device = torch.device("cuda" if torch.cuda.is_available() else "cpu")


class GPT2:
    def __init__(self):
        super(GPT2, self).__init__()

        self.model_type = "GPT2"

        # Load pre-trained model tokenizer (vocabulary)
        self.tokenizer = GPT2Tokenizer.from_pretrained("gpt2")

        # Load pre-trained model (weights)
        self.model = GPT2LMHeadModel.from_pretrained("gpt2")

    def predict_next(self, text):
        # Encode a text inputs
        indexed_tokens = self.tokenizer.encode(text)

        # Convert indexed tokens in a PyTorch tensor
        tokens_tensor = torch.tensor([indexed_tokens])

        # Set the model in evaluation mode to deactivate the DropOut modules
        self.model.eval()

        # If you have a GPU, put everything on cuda
        tokens_tensor = tokens_tensor.to(device)
        self.model.to(device)

        # Predict all tokens
        with torch.no_grad():
            outputs = self.model(tokens_tensor)
            predictions = outputs[0]

        # Get the predicted next sub-word
        probs = predictions[0, -1, :]
        top_next = [self.tokenizer.decode(i.item()).strip() for i in probs.topk(5)[1]]

        return top_next



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from setuptools import find_packages, setup

# The text of the README file
with open('README.md') as readme_file:
    README = readme_file.read()

setup(
    name="next_word_prediction",
    version="0.1.0",
    description="Experiment on the Generative Pretrained Transformer 2 (GPT-2) for Language Modeling task using the PyTorch-Transformers library.",
    long_description=README,
    long_description_content_type="text/markdown",
    url="https://github.com/rdgozum/Next-Word-Prediction",
    author="Ryan Paul Gozum",
    author_email="ryanpaul.gozum@gmail.com",
    license="MIT",
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
   ],
   keywords=["language model"],
   packages=find_packages(),
   install_requires=[],
   package_data={}, # Optional
   data_files=[], # Optional
   entry_points={}, # Optional
   project_urls={}, # Optional
)
